import sys
from pathlib import Path

import yaml
import colorama
from colorama import Fore

import click
from model import Config

README_URL = "https://github.com/laorange/stoken"
BASE_DIR = Path(sys.argv[0]).parent.resolve()
colorama.just_fix_windows_console()


def quit_with_info(info: str = None):
    info = info + '\n\n' if info else ''
    input(f"{info}Press ENTER to exist: ")
    sys.exit(1)


@click.command()
@click.option("--mode", type=click.Choice(['auto', 'hide', 'restore', 'debug'], case_sensitive=False), help='The mode of operation.', default="auto")
@click.option("-e", "--encoding", default="utf-8", help='The encoding used to decode the file.')
@click.option("-p", "--variable-prefix", default="#{{", help="The prefix of variable placeholder.")
@click.option("-s", "--variable-suffix", default="}}#", help="The suffix of variable placeholder.")
@click.option('--debug', is_flag=True, help="In debug mode, `stoken` won't modify files, only detect tokens.")
def entry(mode: str, encoding: str, variable_prefix: str, variable_suffix: str, debug: bool):
    if mode == "debug":
        debug = True
        mode = "auto"

    YAML_PATH = BASE_DIR / "stoken.yaml"
    YAML_PATH = YAML_PATH if YAML_PATH.exists() else BASE_DIR / "stoken.yml"
    if not YAML_PATH.exists():
        quit_with_info(f"{Fore.RED}ERROR: Can't find configuration file! Create `stoken.yaml` in `{BASE_DIR}`. Please refer to {README_URL}.{colorama.Style.RESET_ALL}")
    else:
        print(f"{Fore.GREEN}stoken: read {YAML_PATH}{Fore.RESET}")

    with YAML_PATH.open("rt", encoding="utf-8") as yaml_file:
        try:
            config = Config(**yaml.load(yaml_file, Loader=yaml.SafeLoader))
        except Exception as e:
            quit_with_info(f"{Fore.RED}{e}\n\n"
                           f"Fail to read configuration file. Please refer to {README_URL} and modify the `{YAML_PATH}`.{colorama.Style.RESET_ALL}")

    for file in BASE_DIR.glob("**/*"):
        if file.suffix in config.suffix:
            text_of_this_file = ""
            change_num = 0

            for line_num, line in enumerate(file.open(encoding=encoding)):
                content_of_this_line = line

                for key, token in config.token.items():
                    variable = f"{variable_prefix}{key}{variable_suffix}"

                    if mode == "auto":
                        if token in line:
                            mode = "hide"
                        elif variable in line:
                            mode = "restore"

                    if mode == "hide":
                        if token in line:
                            content_of_this_line = content_of_this_line.replace(token, variable)
                            change_num += 1

                    elif mode == "restore":
                        if variable in line:
                            content_of_this_line = content_of_this_line.replace(variable, token)
                            change_num += 1

                if content_of_this_line != line:
                    print(f"{file.relative_to(BASE_DIR)} line-{line_num + 1}: {Fore.RED}{line.strip()} {Fore.RESET}-> {Fore.GREEN}{content_of_this_line.strip()}{Fore.RESET}")

                text_of_this_file += content_of_this_line

            if change_num:
                if not debug:
                    with file.open("wt", encoding=encoding) as f:
                        f.write(text_of_this_file)
            else:
                print(f"{file.relative_to(BASE_DIR)}: No tokens were found.")

    print(f"{Fore.GREEN}stoken: finished!{Fore.RESET}")


if __name__ == '__main__':
    entry()



from setuptools import setup, find_packages

from pathlib import Path

HERE = Path(__file__).parent
README = (HERE / "README.md").read_text(encoding="utf-8")

setup(
    name="stoken",
    version="0.1.1",
    keywords=["git", "data desensitization", "replace token"],
    description="A code desensitization tool, which can substitute tokens (and other sensitive information) in your code.",
    long_description=README,
    long_description_content_type="text/markdown",
    license="AGPL-3.0",
    url="https://github.com/laorange/stoken",
    project_urls={
        "Bug Tracker": "https://github.com/laorange/stoken/issues",
        "Source Code": "https://github.com/laorange/stoken",
    },
    author="laorange",
    author_email="laorange6666@gmail.com",
    packages=find_packages(),
    include_package_data=True,
    platforms="any",
    install_requires=[
        "pyyaml>=6.0",
        "setuptools>=65.5.0",
        "colorama>=0.4.6",
        "pydantic>=1.10.2",
        "click>=8.1.3",
    ],
    entry_points={
        "console_scripts": [
            "stoken=stoken.main:main"
        ]
    }
)
